from typing import List, Dict, Any, Union

from git import Repo

from src.commit import Commit
from src.summary import Summary


class GitUtils:
    @staticmethod
    def get_commits_between(rep_path, branch, since, after) -> List[Commit]:
        _commits: List[Commit] = []
        _repo = Repo(rep_path)
        for _c in _repo.iter_commits(branch, max_count=300, since=since, after=after):
            _commit = Commit(_c.hexsha, _c.author, _c.committed_datetime, _c.message, _c.stats.files)
            _commits.append(_commit)
        return _commits

    @staticmethod
    def group_by_user(commits, pattern, deep) -> Dict[str, Summary]:
        _summaries_by_user: Dict[Any, Union[Summary, Any]] = {}
        for _commit in commits:
            if _commit.author in _summaries_by_user:
                _summary = _summaries_by_user[_commit.author]
            else:
                _summary = Summary(pattern, deep)
            _summary.add(_commit)
            _summaries_by_user[_commit.author] = _summary
        return _summaries_by_user



from typing import Set, Any, Union


class Summary:
    def __init__(self, pattern, deep):
        self._commits = []
        self._pattern = pattern
        self._deep = deep

    def add(self, commit):
        self._commits.append(commit)

    def show(self) -> str:
        _paths = set()
        for _c in self._commits:
            for _f in _c.files:
                count = _f.count('/')
                if count < 3:
                    _path = _f[0:_f.index('/')]
                else:
                    _path = '/'.join(_f.split("/", self._deep)[0:self._deep])
                _paths.add(_path)
        return '\n'.join(_paths)

    def msg(self) -> str:
        _messages: Set[Union[str, Any]] = set()
        for _c in self._commits:
            match = self._pattern.search(_c.msg)
            if match:
                _short_msg = match.group(0)
            else:
                _short_msg = "NOT FOUND"
            _messages.add(_short_msg)
        return '\n'.join(_messages)

    def hashes(self) -> str:
        _hashes = set()
        for _c in self._commits:
            _hashes.add(_c.hash)
        return '\n'.join(_hashes)



import typing


class Commit:
    def __init__(self, commit_hash, author, commit_datetime, msg, files=[]):
        self._commit_hash = commit_hash
        self._author = author
        self._datetime = commit_datetime
        self._files = files
        self._msg = msg

    def __str__(self):
        return f"{self._datetime} {self._commit_hash} {self._author}"

    @property
    def hash(self) -> str:
        return self._commit_hash

    @property
    def author(self) -> str:
        return self._author

    @property
    def datetime(self) -> str:
        return self._datetime

    @property
    def msg(self) -> str:
        return self._msg

    @property
    def files(self) -> typing.List[str]:
        return self._files


#!/usr/bin/env python
# -*- coding: utf-8 -*-
import setuptools
from version import __version__

with open("README.md", "r", encoding='utf-8') as fh:
    readme = fh.read()

setuptools.setup(
    name="git-history-tools",
    version=__version__,
    author='Orlando Tomás',
    author_email="orlando.tomas@hotmail.com",
    url="https://github.com/orltom/git-history-tools",
    long_description=readme,
    long_description_content_type='text/markdown',
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License"
    ],
    scripts=['githistory'],
    packages=['src'],
    python_requires='>=3.6',
    install_requires=[
        'tabulate',
        'gitpython',
        'docopt'
    ],
)
